//  *****************************************************************************
//                                    main.c
// 
//        Demonstracny program pre dosku KTK ARM7
//
//        Praca s SD kartou pripojenou cez SPI
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"
#include "sd_spi.h"

// Pocet krokov casovaca na 1 us
#define STEPS   ((unsigned int) ((MCK/8) + 500000) / 1000000L)

//  *****************************************************
//  Mikrosekundove oneskorenie (max. cca 10 000)
//  *****************************************************
void delay(unsigned int us)
{
  AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC0);  // povolenie hodin pre TC0
  AT91C_BASE_TC0->TC_CMR = 1;                      // volba hodin (MCK/8)
  AT91C_BASE_TC0->TC_RC = us * STEPS;              // nastavenie porovnavacieho registra C
  AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN |        // povolenie hodin
                           AT91C_TC_SWTRG;         // restart casovaca

  // cakanie na ukoncenie oneskorenia
  // ked je nastaveny compare C flag, je hotovo
  while ((AT91C_BASE_TC0->TC_SR & AT91C_TC_CPCS) == 0) asm ("nop");
    
  AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_TC0);  // zakazanie hodin pre TC0
}

//  *******************************************************
//  Milisekundove cakanie
//  *******************************************************
void cakaj(unsigned int ms)
{ unsigned int ii;
 
  for(ii=0; ii<ms; ii++) 
    delay(1000);
}

//  *******************************************************
//  Blikanie LED - indikacia chyby 
//  blika v seriach podla danej hodnoty
//  *******************************************************
void blikaj(int pocet)
{ int i;

  // zakazanie preruseni
  AT91C_BASE_AIC->AIC_IDCR = 0xffff;

  // zaciatocny stav
  AT91C_BASE_PIOA->PIO_CODR = D1;	      // vypnutie LED

  // blikanie    
  while(1)
  { for(i=0;i<pocet;i++)
    { AT91C_BASE_PIOA->PIO_SODR = D1;	      // zapnutie LED
      cakaj(100);
      AT91C_BASE_PIOA->PIO_CODR = D1;	      // vypnutie LED
      cakaj(100);
    }
    // medzera medzi seriami
    cakaj(400);
  }
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main (void) 
{ int ii, pom;
  unsigned char buf[512];

  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);

  //******************* Inicializacia portu A ******************
  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
  // konfiguracia jednotlivych pinov
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = D1;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = D1;
  // vsetko je vypnute
  AT91C_BASE_PIOA->PIO_CODR = D1;

  //****************** Inicializacia SPI a SD karty ************
  // inicializacia SPI
  SPI_init();
  // inicializacia SD karty
  if ((pom = SD_SPI_init()) != 0)
    blikaj(pom);     // indikacia chyby - treba restart

  //******************* Hlavna cast programu *******************

  // zapnutie LED - presla uvodna faza inicializacie
  AT91C_BASE_PIOA->PIO_SODR = D1;

  // priprava udajov na zapisanie
  for(ii=0;ii<512;ii++)
    buf[ii] = (unsigned char) ii;

  // zapis niekolko stranok do SD karty
  for(ii=0;ii<100;ii++)
    if (SD_Write(ii,buf) != 0)
      blikaj(3);

  // precitanie stranky z karty pre kontrolu
  if (SD_Read(2,buf) != 0)
    blikaj(4);

  // kontrola nacitanych udajov
  for(ii=0;ii<512;ii++)
    if (buf[ii] != (unsigned char) ii)
      blikaj(5);
  
  // vsetko v poriadku - zhasne LED
  AT91C_BASE_PIOA->PIO_CODR = D1;

  // nekonecna slucka
  while(1);
}
